"""
addons/eventscripts/smauth_ess/smauth_ess.py
Version 1.00.000

Licensed under:
http://creativecommons.org/licenses/by-nc-nd/3.0/

If wish to alter the file/contents or reuse some parts please contact me.

(C) [#OMEGA] - K2 2010-2011
"""

# ##### #
# Psyco #
# ##### #

import psyco
psyco.full()

# #################### #
# Eventscripts Imports #
# #################### #

# ES Imports

import es
import smauth

# Custom ES Imports

import pubcvarlib

# ############## #
# Python Imports #
# ############## #

# ########### #
# Script Info #
# ########### #

info = es.AddonInfo()
info.name = "Sourcemod ESS Authprovider"
info.version = "1.00.000"
info.author = "[#OMEGA] - K2"
info.url = "http://www.german-slaughterhouse.de"
info.description = "Provides auth access to sm files."
info.tags = "sm sourcemod auth provider"

# ############# #
# Load & Unload #
# ############# #

def load():
	# Public Vars
	pubcvarlib.Register('es_smauth_ess_ver', info.version)
	# Init Command Handler
	global MyCommandHandler
	MyCommandHandler = CommandHandlerClass()
	# Register command
	for command in ['smauth','es_smauth','es_xsmauth', 'xsmauth']:
		es.regcmd(command,'smauth_ess/CommandHandler')
	
def unload():
	# Public Vars
	pubcvarlib.Unregister('es_smauth_ess_ver')
	
# ######### #
# Functions #
# ######### #

def CommandHandler():
	MyCommandHandler()

class CommandHandlerClass(object):
	def __init__(self):
		self.subcommands = {
			'flagtolevel': {
				'func': smauth.FlagToLevel,
				'syntax': 'smauth flagtolevel <variable> <flag>'
			},
			'leveltoflag': {
				'func': smauth.LevelToFlag,
				'syntax': 'smauth leveltoflag <variable> <level>'
			},
			'isvalidflag': {
				'func': smauth.IsValidFlag,
				'syntax': 'smauth isvalidflag <variable> <flag>'
			},
			'isvalidgroup': {
				'func': smauth.IsValidGroup,
				'syntax': 'smauth isvalidgroup <variable> <group>'
			},
			'isvalidlevel': {
				'func': smauth.IsValidLevel,
				'syntax': 'smauth isvalidflag <variable> <level>'
			},
		}
		self.playersubs = {
			'isuser': {
				'func': smauth.PlayerAccess.IsUser,
				'syntax': 'smauth player isuser <variable> <userid>'
			},
			'isadmin': {
				'func': smauth.PlayerAccess.IsAdmin,
				'syntax': 'smauth player isadmin <variable> <userid>'
			},
			'isroot': {
				'func': smauth.PlayerAccess.IsRoot,
				'syntax': 'smauth player isuser <variable> <userid>'
			},
			'hasflag': {
				'func': smauth.PlayerAccess.HasFlag,
				'syntax': 'smauth player hasflag <variable> <userid> <flag>'
			},
			'hasflags': {
				'func': smauth.PlayerAccess.HasFlags,
				'syntax': 'smauth player hasflags <variable> <userid> <flags>'
			},
			'getimmunity': {
				'func': smauth.PlayerAccess.GetImmunity,
				'syntax': 'smauth player getimmunity <variable> <userid>'
			},
			'ingroup': {
				'func': smauth.PlayerAccess.InGroup,
				'syntax': 'smauth player ingroup <variable> <userid> <group>'
			},
			'cantarget': {
				'func': smauth.PlayerAccess.CanTarget,
				'syntax': 'smauth player cantarget <variable> <userid> <target userid>'
			},
		}
		
		self.__SetCommandData(self.subcommands)
		self.__SetCommandData(self.playersubs)
	def __call__(self):
		# Expand es variables
		if es.getargv(0) in ['es_smauth','smauth']:
			cmd = 'es es_xsmauth '
			for i in range(1,es.getargc()):
				cmd += '"' + es.getargv(i) + '"'
			es.ServerCommand(cmd)
		else:
			self.subcommand = es.getargv(1).lower()
			if self.subcommand in self.subcommands:
				if self.__CheckParams(self.subcommands):
					self.__RunCommand()
			elif self.subcommand == 'player':
				self.subcommand = es.getargv(2).lower()
				if self.subcommand in self.playersubs:
					if self.__CheckParams(self.playersubs):
						self.__RunCommand(3)
				elif self.subcommand == 'help':
					self.__DisplayHelp(self.playersubs,3)
				else:
					self.__Msg('Invalid subcommand!')
					self.__Msg('Valid subcommands are %s' % ' '.join(list(self.playersubs)) + ' help')
			elif self.subcommand == 'help':
				self.__DisplayHelp(self.subcommands,2)
			else:
				self.__Msg('Invalid subcommand!')
				self.__Msg('Valid subcommands are %s' % ' '.join(list(self.subcommands)) + ' player help')
	def __DisplayHelp(self, cmds, offset):
		if es.getargv(offset) in cmds:
			for item in cmds[es.getargv(offset)]['help']:
				self.__Msg(item)
		else:
			self.__Msg('Syntax: smauth [player] help <subcommand>')
			self.__Msg('Valid subcommands are %s' % ' '.join(list(cmds)))
	def __SetCommandData(self, cmds):
		for command in cmds:
			# Make it prettier in console
			helplines = cmds[command]['func'].__doc__.split('\n')
			for i in xrange(0, len(helplines)):
				helplines[i] = helplines[i].lstrip('\t')
			helplines.pop(len(helplines) - 1)
			
			cmds[command]['help'] = ['Help for command ' + command + ':', '', 
				'Syntax: ' + cmds[command]['syntax']] + helplines
				
				
			syntax = cmds[command]['syntax'].split(' ')
			cmds[command]['args'] = len(syntax) - 1
			cmds[command]['voffset'] = syntax.index('<variable>')
			try:
				cmds[command]['aoffset'] = syntax.index('<userid') + 1
			except ValueError:
				cmds[command]['aoffset'] = syntax.index('<variable>') + 1
	def __FlagToLevel(self):
		es.set(es.getargv(2),smauth.FlagToLevel(es.getargv(3)))
	def __LevelToFlag(self):
		smauth.LevelToFlag(es.getargv(3))
	def __CheckParams(self, cmds):
		for i in xrange (1,cmds[self.subcommand]['args']):
			if not len(es.getargv(i)):
				self.__Msg('Syntax: %s' % cmds[self.subcommand]['syntax'])
				return False
		return True
	def __RunCommand(self, offset=2):
		var = es.getargv(offset)
		if self.__CheckVar(es.getargv(offset)):
			# 2 are regular commands
			# 3 are player specific commands
			if offset == 2:
				self.__SetVar(self.subcommands)
			elif offset == 3:
				userid = es.getargv(4)
				if es.exists('userid',userid):
					self.player = smauth.PlayerAccess(userid)
					self.__SetVar(self.playersubs, True)
				else:
					self.__Msg('ERROR: Userid does not exist.')
					self.__Msg('The var %s could not be set.' % var)
	def __CheckVar(self, var):
		if es.exists('variable',var):
			return True
		else:
			self.__Msg('ERROR: Variable does not exist.')
			self.__Msg('The var %s could not be set.' % var)
			return False
	def __SetVar(self, cmds, playercmd=False):
		voffset = cmds[self.subcommand]['voffset']
		
		args = [self.player, ] if playercmd else []
		for i in xrange(cmds[self.subcommand]['aoffset'],cmds[self.subcommand]['args']):
			args.append(es.getargv(i))
		
		result = cmds[self.subcommand]['func'](*args)
		
		if isinstance(result, bool):
			result = int(result)
		
		es.set(es.getargv(voffset), result)
	def __Msg(self, msg):
		es.dbgmsg(0,'[smauth] ' + msg)
		